define(["dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/on",
    "dojo/topic",
    "dojo/dom",
    "dojo/dom-style",
    "dojo/dom-construct",
    "com/alta4/utility/SqlGen",
    "com/alta4/atlas/extensions/AtlasExternalInterface"
], function (declare,
        lang,
        on,
        topic,
        dom,
        domStyle,
        domConstruct,
        sqlGen,
        atlasExternalInterface) {
    return declare(null, {
        count: 0,
        filterElements: null,
        config: null,
        visibleFeatures: null,
        disableUpdate: false,
        constructor: function (config) {
            this.config = config;
            atlas.mapReady(lang.hitch(this, function () {
                if (atlas.model.map.layerstackComplete) {
                    this.layerstackComplete();
                } else {
                    topic.subscribe(atlas.model.map.controller.ATLAS_MAP_LAYERSTACK_COMPLETE, lang.hitch(this, function () {
                        setTimeout(lang.hitch(this, function () {
                            this.layerstackComplete();
                        }), 10);
                    }));
                }
            }));
        },
        layerstackComplete: function () {
            var pseudoLayer = this.config.enableUniqueValueRenderer && atlas.model.toc.getPseudoLayer(atlas.model.map.mapLayers, this.config.layerId, this.config.filterField);
            if (pseudoLayer) {
                this.config.pseudoLayer = pseudoLayer;
                this.config.filterValues = [];
                for (var i = 0; i < pseudoLayer.children.length; i++) {
                    var child = pseudoLayer.children[i];
                    this.config.filterValues.push({
                        key: child.uniqueValue,
                        text: child.name,
                        checked: child.tocItem.checkbox.checked
                    });
                }
            } else {
                // Convert number values if necessary
                var layer = atlas.model.map.getLayerDefinitionByMapLayerId(this.config.layerId);
                if(layer && this._isNumberLayer(layer)) {
                    var values = this.config.filterValues;
                    for (var i = 0; i < values.length; i++) {
                        values[i].key = parseFloat(values[i].key);
                    }
                }
            }
            this.setup();
            var checked = this.config.filtersCheckedOnStart;
            if(checked == "all" || checked == "none") {
                this.disableUpdate = true;
                checked = checked == "all";
                for (var i = 0; i < this.filterElements.length; i++) {
                    var element = this.filterElements[i];
                    if(element.checked != checked)
                        element.click();
                }
                this.disableUpdate = false;
            }
            this.update();
            topic.subscribe(atlas.events.ATLAS_LAYER_FIELD_FILTER_UPATE, lang.hitch(this, this.updateLayerFieldFilter));
        },
        _isNumberLayer: function(layer) {
            var jsonLayer = layer.jsonLayer;
            var fields = jsonLayer.datasource.fields;
            if(fields) {
                var numberTypes = [
                    "esriFieldTypeSmallInteger",
                    "esriFieldTypeInteger",
                    "esriFieldTypeOID"
                ];
                for(var i=0; i<fields.length; i++) {
                    if(fields[i].name === this.config.filterField) {
                        if(numberTypes.indexOf(fields[i].type) >= 0) {
                            return true;
                        }
                        return false;
                    }
                }
            }
            return false;
        },
        updateLayerFieldFilter: function (layerId, fieldName, filter, finalWhere) {
            if (layerId == this.config.layerId && fieldName == this.config.filterField) {
                var pseudoLayer = this.config.pseudoLayer;
                this.disableUpdate = true;
                try {
                    var filterType = filter ? filter.type : 'TRUE';
                    var selected = false, element = null, key = null, i=0;
                    switch (filterType) {
                        case 'TRUE':
                            selected = true;
                        case 'FALSE':
                            for (i = 0; i < this.filterElements.length; i++) {
                                element = this.filterElements[i];
                                if (selected != element.checked) {
                                    element.checked = selected;
                                }
                            }
                            break;
                        case 'IN':
                            for (i = 0; i < this.filterElements.length; i++) {
                                element = this.filterElements[i];
                                key = this.config.filterValues[i].key;
                                if (pseudoLayer && key === 'othersValue') {
                                    if(pseudoLayer.othersValues.length) {
                                        // wenn ein "sonstige"-Wert im Filter, dann alle
                                        selected = filter.values.indexOf(pseudoLayer.othersValues[0]) >= 0;
                                    } else {
                                        selected = false;
                                    }
                                } else {
                                    selected = filter.values.indexOf(key) >= 0;
                                }
                                if (selected != element.checked) {
                                    element.checked = selected;
                                }
                            }
                            break;
                        default:
                            console.error('FilterInput unhandled filter type: ' + filter.type);
                            break;
                    }
                } finally {
                    this.disableUpdate = false;
                }
            }
        },
        setup: function () {
            var mapFilter = dom.byId('mapFilter');
            domConstruct.create("h3", {}, mapFilter).textContent = this.config.filterText;

            var table = domConstruct.create("table", {}, mapFilter);
            this.filterElements = new Array();
            var filterValues = this.config.filterValues;
            var column = 3;
            var tr = null;
            for (var i = 0; i < filterValues.length; i++) {
                var filterValue = filterValues[i];

                if (column == 3) {
                    tr = domConstruct.create("tr", {}, table);
                    column = 1;
                } else {
                    column++;
                }

                var id = "mapFilterValue" + i;
                var td = domConstruct.create("td", {"class": "mapFilterEntry"}, tr);
                var input = domConstruct.create("input", {"type": "checkbox", value: filterValue.key, id: id, checked: filterValue.checked}, td);
                domConstruct.create("label", {"class": "mapFilterCheckboxLabel", "for": id}, td).textContent = filterValue.text;
                on(input, "change", lang.hitch(this, this.update));
                this.filterElements.push(input);
            }
            for (; column < 3; column++) {
                domConstruct.create("td", {"class": "mapFilterEntry"}, tr);
            }

            domConstruct.create("div", {style: "clear: both;"}, mapFilter);
            if(this.config.showFilter)
                domStyle.set(mapFilter, 'display', '');

            atlas.model.map.reposition();
            atlas.model.map.resize(true);
        },
        update: function () {
            if (!this.disableUpdate) {
                var hasOthers = false;
                var whitelist = [];
                for (var i = 0; i < this.filterElements.length; i++) {
                    var element = this.filterElements[i];
                    var key = this.config.filterValues[i].key;
                    if(element.checked) {
                        whitelist.push(key);
                        if (key === 'othersValue') {
                            hasOthers = true;
                        }
                    }
                }
                if(hasOthers && this.config.pseudoLayer) {
                    whitelist = whitelist.concat(this.config.pseudoLayer.othersValues);
                }
                
                var filter;
                if (whitelist.length) {
                    filter = sqlGen.IN(this.config.filterField, whitelist);
                } else {
                    // NONE, Layer per Query ausgeschaltet
                    filter = sqlGen.FALSE;
                }
                atlasExternalInterface.setLayerFieldFilter(this.config.layerId, this.config.filterField, filter);
            }
        }
    });
});