define(["dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/on",
    "dojo/topic",
    "dojo/aspect",
    "dojo/request",
    "dojo/dom",
    "dojo/dom-style",
    "dojo/dom-geometry",
    "dojox/grid/DataGrid",
    "dojo/data/ItemFileWriteStore",
    "com/alta4/utility/ErrorHandler",
    "com/alta4/utility/Utility",
    "com/alta4/EsriLoader!esri/geometry/Point",
    "com/alta4/EsriLoader!esri/geometry/Polygon",
    "com/alta4/EsriLoader!esri/geometry/Polyline",
    "./Events"
], function (declare,
        lang,
        on,
        topic,
        aspect,
        request,
        dom,
        domStyle,
        domGeometry,
        DataGrid,
        ItemFileWriteStore,
        errorHandler,
        utility,
        Point,
        Polygon,
        Polyline,
        events) {
    return declare(null, {
        constructor: function (config) {
            this.config = config;
            this.featureMap = {};
            this.visibleFeatures = {};
            this.requestInProgress = {};
            this.validKeys = {};
            this.visibleFeatures = {};
            
            this.setup();
            
            topic.subscribe(atlas.events.ATLAS_LAYER_FIELD_FILTER_UPATE, lang.hitch(this, this.updateLayerFieldFilter));
            atlas.mapReady(lang.hitch(this, this.requestLayerFeatures));
            topic.subscribe(atlas.events.ATLAS_QUERY_VECTOR_RESULT, lang.hitch(this, this.onQueryVectorResult));
        },
        setup: function() {
            var filteredTable = dom.byId('filteredTable');
            domStyle.set(filteredTable, 'display', '');
            var header = [];
            this.resultOutFields = [];
            var columns = this.config.tableColumns;
            var columnWidths = [];
            var totalColumnWidths = 0;
            var autoColumns = 0;
            for (var i = 0; i < columns.length; i++) {
                var width = parseInt(columns[i].width);
                if(width) {
                    columnWidths.push(width + 'px');
                    totalColumnWidths += width;
                } else {
                    columnWidths.push(null);
                    autoColumns++;
                }
            }

            var tableWidth = domGeometry.position(filteredTable).w - 20 - totalColumnWidths;
            var autoColWidth = Math.floor(autoColumns ? (tableWidth/autoColumns) : 0) + 'px';
            for (var i = 0; i < columns.length; i++) {
                var width = columnWidths[i] || autoColWidth;
                header.push({'name': columns[i].text, 'field': columns[i].key, width: width});
                this.resultOutFields.push(columns[i].key);
            }
            this.resultOutFields = this.resultOutFields.join(',');
            
            this.grid = new DataGrid({
                store: null,
                structure: [header],
                sortInfo: -2,
                rowsPerPage: 50
            });
            aspect.after(this.grid, 'onRowClick', lang.hitch(this, this.onRowClick));
            aspect.after(this.grid, 'onMouseOverRow', lang.hitch(this, this.onMouseOverRow));
            aspect.after(this.grid, 'onMouseOutRow', lang.hitch(this, this.onMouseOutRow));

            this.grid.placeAt(filteredTable);
            this.grid.startup();
            
            // Resize grid on window resize
            on(window, 'resize', lang.hitch(this, function() {
                var header = [];
                var tableWidth = domGeometry.position(filteredTable).w - 20 - totalColumnWidths;
                var autoColWidth = Math.floor(autoColumns ? (tableWidth/autoColumns) : 0) + 'px';
                for (var i = 0; i < columns.length; i++) {
                    var width = columnWidths[i] || autoColWidth;
                    header.push({'name': columns[i].text, 'field': columns[i].key, width: width});
                }
                this.grid.setStructure([header]);
                this.grid.resize();
                this.grid.update();
            }));
        },
        updateLayerFieldFilter: function (layerId, fieldName, filter, finalWhere) {
            if (layerId == this.config.layerId) {
                var url = atlasHelper.restUrl("/MapServer/" + layerId + "/query");
                atlas.handleRequest(
                        request.get(url, {
                            handleAs: "json",
                            query: {
                                f: "json",
                                outSR: atlas.model.map.spatialReference.wkid,
                                where: finalWhere
                            }
                        }),
                        lang.hitch(this, function (data) {
                            topic.publish(atlas.events.ATLAS_MAP_HIDE_BUSY, layerId);
                            var visibleFeatures = {};
                            for (var i = 0; i < data.features.length; i++) {
                                var feature = data.features[i];
                                var key = feature.attributes.layerid + "/" + feature.attributes.id;
                                visibleFeatures[key] = true;
                            }
                            this.updateVisibleFeatures(visibleFeatures);
                        }),
                        function (data) {
                            topic.publish(atlas.events.ATLAS_MAP_HIDE_BUSY, layerId);
                            errorHandler.showWarning("ERROR in retrieving Features: " + data.message);
                            utility.remotingError('atlasFX Typo3 Plugin', data);
                        });
            }
        },
        onQueryVectorResult: function (layerDefinition, featureSet) {
            if (layerDefinition && layerDefinition.esriLayer) {
                var layerId = layerDefinition.getLayerId();
                if (layerId == this.config.layerId) {
                    // update graphics
                    var featureMap = this.featureMap;
                    for (var key in featureMap) {
                        featureMap[key].graphic = null;
                    }
                    var graphics = layerDefinition.esriLayer.graphics;
                    for (var i = 0; i < graphics.length; i++) {
                        var g = graphics[i];
                        var key = layerId + "/" + g.id;
                        var feature = featureMap[key];
                        if (feature) {
                            feature.graphic = g;
                            if (feature == this.infoWindowFeature && g.highlightSymbol) {
                                g.click();
                                topic.publish(events.FEATURE_HIGHLIGHT, feature);
                                this.infoWindowFeature = null;
                            }
                        }
                    }
                }
            }
        },
        requestLayerFeatures: function () {
            var layerId = this.config.layerId;
            var url = atlasHelper.restUrl("/MapServer/" + layerId + "/query");

            topic.publish(atlas.events.ATLAS_MAP_SHOW_BUSY, layerId);
            atlas.handleRequest(
                    request.get(url, {
                        handleAs: "json",
                        query: {
                            f: "json",
                            outFields: this.resultOutFields,
                            outSR: atlas.model.map.spatialReference.wkid
                        }
                    }),
                    lang.hitch(this, function (data) {
                        topic.publish(atlas.events.ATLAS_MAP_HIDE_BUSY, layerId);
                        this.allFeatures = data.features;
                        for (var i = 0; i < data.features.length; i++) {
                            var feature = data.features[i];
                            var key = feature.attributes.layerid + "/" + feature.attributes.id;
                            feature.key = key;
                            
                            var center, geometry;
                            switch(feature.geometry.geometryType) {
                                case "esriGeometryPolygon":
                                    geometry = new Polygon({rings: feature.geometry.rings, spatialReference: atlas.model.map.spatialReference});
                                    center = geometry.getExtent().getCenter();
                                    break;
                                case "esriGeometryPolyline":
                                    geometry = new Polyline({paths: feature.geometry.paths, spatialReference: atlas.model.map.spatialReference});
                                    center = geometry.getExtent().getCenter();
                                    break;
                                default:
                                    center = new Point(feature.geometry.x, feature.geometry.y, atlas.model.map.spatialReference);
                                    break;
                            }
                            this.featureMap[key] = {
                                id: feature.attributes.id,
                                key: key,
                                attributes: feature.attributes,
                                point: center
                            };
                        }
                        this.updateStore();
                    }),
                    lang.hitch(this, function (data) {
                        topic.publish(atlas.events.ATLAS_MAP_HIDE_BUSY, layerId);
                        errorHandler.showWarning("ERROR in retrieving Features: " + data.message);
                        utility.remotingError('Kindergaerten', data);
                    }));
        },
        onRowClick: function(u,args) {
            var e = args[0];
            if(e.rowIndex >= 0) {
                var item = this.grid.getItem(e.rowIndex);
                var feature = this.featureMap[item.key];
                this.infoWindowFeature = feature;
                var scale = atlas.model.map.getAtlasMapObject().searchResultScale;
                if(scale == 0)
                    atlas.model.map.centerAt(feature.point);
                else
                    atlas.model.map.centerAndScale(feature.point, scale);
                setTimeout(lang.hitch(this, function() {
                    if(this.infoWindowFeature && this.infoWindowFeature.graphic) {
                        this.infoWindowFeature.graphic.click();
                        this.infoWindowFeature = null;
                    }
                }), 1000);
            }
        },
        onMouseOverRow: function(u,args) {
            var e = args[0];
            if(e.rowIndex >= 0) {
                var item = this.grid.getItem(e.rowIndex);
                topic.publish(events.FEATURE_HIGHLIGHT, this.featureMap[item.key]);
            }
        },
        onMouseOutRow: function(u,args) {
            var e = args[0];
            if(e.rowIndex >= 0) {
                topic.publish(events.FEATURE_UNHIGHLIGHT);
            }
        },
        updateStore: function() {
            if(this.grid && this.allFeatures) {
                var dataStore = {identifier: "key", items: []};
                for (var i = 0; i < this.allFeatures.length; i++) {
                    var feature = this.allFeatures[i];
                    if (feature.key in this.visibleFeatures) {
                        dataStore.items.push(lang.mixin({
                            key: feature.key
                        }, feature.attributes));
                    }
                }
                this.grid.setStore(new ItemFileWriteStore({data: dataStore}));
            }
        },
        updateVisibleFeatures: function (visibleFeatures) {
            this.visibleFeatures = visibleFeatures;
            this.updateStore();
        }
    });
});