define(["dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/request",
    "dojo/topic",
    "dojo/aspect",
    "com/alta4/EsriLoader!esri/graphic",
    "com/alta4/EsriLoader!esri/geometry/Point",
    "com/alta4/EsriLoader!esri/symbols/PictureMarkerSymbol",
    "./FilterInput",
    "./FilterTable",
    "./Events"
], function (declare,
        lang,
        request,
        topic,
        aspect,
        Graphic,
        Point,
        PictureMarkerSymbol,
        FilterInput,
        FilterTable,
        events) {
    return declare(null, {
        count: 0,
        constructor: function (config) {
            request(atlasHelper.baseUrl()
                    + "/spring/rest/license/products/ATLASFX_TYPO3_MAP_FILTER_PLUGIN", {
                        handleAs: "json"
                    }).then(
                    lang.hitch(this, function (response) {
                        if (response.isLicensed) {
                            if (config.enableResultTable) {
                                this.filterTable = new FilterTable(config);
                            }
                            this.filterInput = new FilterInput(config);

                            topic.subscribe(events.FEATURE_HIGHLIGHT, lang.hitch(this, this.onFeatureHighlight));
                            topic.subscribe(events.FEATURE_UNHIGHLIGHT, lang.hitch(this, this.onFeatureUnhighlight));

                            atlas.mapReady(lang.hitch(this, this.onMapReady));
                        } else {
                            // keine Berechtigung, fraud management
                        }
                    })
            );
        },
        onFeatureHighlight: function (feature) {
            var graphic = feature.graphic;
            if (graphic && graphic.highlightSymbol)
                this.showCurrentGraphic(graphic);

            this.highlightGraphic.setGeometry(feature.point);
            this.highlightGraphic.show();
        },
        onFeatureUnhighlight: function () {
            this.hideCurrentGraphic();
        },
        onMapReady: function () {
            var red_glow = atlasHelper.jsUrl('/images/searchBox/red_glow.gif');
            var highlightSymbol = PictureMarkerSymbol(red_glow, 32, 32);
            this.highlightGraphic = new Graphic(new Point(0, 0, atlas.model.map.spatialReference), highlightSymbol);
            atlas.model.map.graphics.add(this.highlightGraphic);
            this.highlightGraphic.hide();

            aspect.after(atlas.model.map.infoWindow, 'onHide', lang.hitch(this, this.hideCurrentGraphic));
        },
        hideCurrentGraphic: function () {
            if (!atlas.model.map.infoWindow.isShowing) {
                if (this.currentGraphic) {
                    this.currentGraphic.setSymbol(this.currentGraphic.normalSymbol);
                    this.currentGraphic = null;
                }
                this.highlightGraphic.hide();
            }
        },
        showCurrentGraphic: function (graphic) {
            graphic.setSymbol(graphic.highlightSymbol);
            this.currentGraphic = graphic;
        }
    });
});