define(["../Utility",
	"jquery"
], function (
		utility,
		$) {
	function getIndex(element) {
		var i = 0;
		while( (element = element.previousSibling) !== null ) 
		  i++;
		return i;
	}
	return utility.createClass(null, {
		constructor: function (config) {
			this.config = config;
			if(!config.labelFormatter)
				config.labelFormatter =  function(id) { return id; };
			this.placeholder = utility.createElement('li', {'class': "a4DragListPlaceholder"});
			this.emptyDummy = utility.createElement('li', {'class': "hidden"});
			this.makeDropTarget(this.placeholder);

			this.domNode = utility.createElement('div', {'class': 'a4DragList'}, config.parentElement);
			
			if(config.doubleList)
				this.createDoubleList(config);
			else
				this.createSingleList(config);
		},
		destroy: function() {
			$(this.domNode).remove();
		},
		createSingleList: function(config) {
			this.selectedContainer = this.createDragListContainer(config.labelSingle);
			this.updateSingleList(config.items);
		},
		updateSingleList: function(items) {
			var NodeClass = this.config.NodeClass;
			var labelFormatter = this.config.labelFormatter;
			var extraParam = this.config.extraParam;
			
			var selectedNodes = [];
			for (var i = 0; i < items.length; i++) {
				var item = items[i];
				var node = new NodeClass(item, labelFormatter, extraParam);
				selectedNodes.push(node.domNode);
			}
			
			this.setNodes(this.selectedContainer, selectedNodes);
		},
		createDoubleList: function(config) {
			$(this.domNode).addClass('doubleList');
			
			this.possibleContainer = this.createDragListContainer(config.labelPossible);
			this.selectedContainer = this.createDragListContainer(config.labelChosen);

			var buttons = utility.createElement('div', {}, this.domNode);
			var addAll = utility.createElement('button', {type: 'button', 'class': 'fieldButton btn btn-default'}, buttons);
			addAll.textContent = 'Alle hinzufügen';
			$(addAll).on('click', function () {
				this.moveAllNodes(this.possibleContainer, this.selectedContainer);
				this.onChange();
			}.bind(this));

			var removeAll = utility.createElement('button', {type: 'button', 'class': 'fieldButton btn btn-default'}, buttons);
			removeAll.textContent = 'Alle entfernen';
			$(removeAll).on('click', function () {
				this.moveAllNodes(this.selectedContainer, this.possibleContainer);
				this.onChange();
			}.bind(this));
			
			this.updateDoubleList(config.items, config.allIds);
		},
		updateDoubleList: function(items, allIds) {
			var NodeClass = this.config.NodeClass;
			var labelFormatter = this.config.labelFormatter;
			var extraParam = this.config.extraParam;
			var dataFactory = this.config.dataFactory;
			
			var restIds = [];
			for(var i=0; i<allIds.length; i++) 
				restIds.push('' + allIds[i]);
			var possibleNodes = [];
			var selectedNodes = [];
			for (var i = 0; i < items.length; i++) {
				var item = items[i];
				var node = new NodeClass(item, labelFormatter, extraParam);
				var id = node.getId();
				var index = restIds.indexOf(id);
				if(index >= 0) {
					restIds.splice(index, 1);
					selectedNodes.push(node.domNode);
				}
			}
			for (var i = 0; i < restIds.length; i++) {
				var id = restIds[i];
				var node = new NodeClass(dataFactory(id), labelFormatter, extraParam);
				possibleNodes.push(node.domNode);
			}
			this.setNodes(this.possibleContainer, possibleNodes);
			this.setNodes(this.selectedContainer, selectedNodes);
		},
		setNodes: function(list, nodes) {
			$(list).empty();
			for(var i=0; i<nodes.length; i++) {
				utility.placeElement(nodes[i], list);
				this.linkItem(nodes[i]);
			}
		},
		onDragEnd: function() {
			if (this.dragging) {
				$(this.dragging).removeClass('a4DraggedNode');
				$(this.dragging).removeClass('hidden');
				if (this.placeholder.parentElement)
					this.placeholder.parentElement.removeChild(this.placeholder);
				if (this.emptyDummy.parentElement)
					this.emptyDummy.parentElement.removeChild(this.emptyDummy);
				this.dragging = null;
				this.onChange();
			}
		},
		makeDropTarget: function(item, isList) {
			$(item).on('dragover dragenter drop', function (e) {
				if(!this.dragging)
					return true;
				if (this.dragging.parentElement !== this.selectedContainer
						&& this.dragging.parentElement !== this.possibleContainer) {
					return true;
				}
				if (e.type === 'drop') {
					e.preventDefault();
					e.stopPropagation();
					utility.placeElement(this.dragging, this.placeholder, 'after');
					this.onDragEnd();
					return false;
				}
				e.preventDefault();
				e.stopPropagation();
				e.originalEvent.dataTransfer.dropEffect = 'move';
				if (e.type !== 'dragover') {
					if (isList) {
						utility.placeElement(this.placeholder, item);
					} else {
						var where = getIndex(this.placeholder) < getIndex(item) ? 'after' : 'before';
						utility.placeElement(this.placeholder, item, where);
					}
					if(!$(this.dragging).hasClass('hidden')) {
						var height = this.dragging.clientHeight;
						$(this.placeholder).css('height', height + 'px');
						$(this.dragging).addClass('hidden');
						utility.placeElement(this.emptyDummy, this.dragging, 'after');
					}
				}
				return false;
			}.bind(this));
		},
		linkItem: function(item) {
			var dragThumb = item.querySelector('.dragListNodeLabel');
			dragThumb.setAttribute('draggable', 'true');
			$(dragThumb).on('dragstart', function (e) {
				var dt = e.originalEvent.dataTransfer;
				dt.effectAllowed = 'move';
				dt.setData('text', 'text');
				this.dragging = item;
				$(this.dragging).addClass('a4DraggedNode');
			}.bind(this));
			$(dragThumb).on('dragend', this.onDragEnd.bind(this));
			this.makeDropTarget(item);
		},
		createSortButton: function(code, title, parent) {
			return utility.createElement('span', {
				innerHTML: '&#x' + code +';',
				title: title,
				'class': 'sortButton btn btn-default'
			}, parent);
		},
		createDragListContainer: function (text) {
			var container = utility.createElement('div', {"class": "a4DragListContainer"}, this.domNode);
			var h3 = utility.createElement('h3', {textContent: text}, container);
			var upArrow = this.createSortButton('25bc', 'Absteigend sortieren', h3);
			var downArrow = this.createSortButton('25b2', 'Aufsteigend sortieren', h3);

			var list = utility.createElement('ol', {}, container);
			$(upArrow).on('click', function() { this.sortList(list, true);}.bind(this));
			$(downArrow).on('click', function() { this.sortList(list, false);}.bind(this));
			this.makeDropTarget(list, true);
			return list;
		},
		sortList: function(container, descending) {
			var children = container.children;
			var nodes = [];
			for(var i=0; i<children.length; i++)
				nodes.push(children[i]);

			nodes.sort(function(a, b) {
				var aLower = a.textContent.toLowerCase();
				var bLower = b.textContent.toLowerCase();
				if(aLower < bLower)
					return -1;
				else if(aLower > bLower)
					return 1;
				return 0;
			});
			if(descending)
				nodes.reverse();
			for (i = 0; i < nodes.length; ++i)
				container.appendChild(nodes[i]);
		},
		onChange: function() {
			if(this.config.onChange)
				this.config.onChange();
		},
		onDrop: function () {
			this.onChange();
		},
		moveAllNodes: function (from, to) {
			while(from.children.length > 0)
				to.appendChild(from.children[0]);
		},
		getData: function () {
			var items = [];
			var nodeElements = this.selectedContainer.children;
			for (var i = 0; i < nodeElements.length; i++) {
				var nodeElement = nodeElements[i];
				var node = nodeElement.a4Model;
				items.push(node.data);
			}
			return items;
		}
	});
});
